@@ -1,5 +1,6 @@
 from pyield.bc.auction import auctions
-from pyield.bc.bcdata import (
+from pyield.bc.ptax_api import ptax, ptax_series
+from pyield.bc.rates import (
     di_over,
     di_over_series,
     selic_over,
@@ -7,7 +8,6 @@ from pyield.bc.bcdata import (
     selic_target,
     selic_target_series,
 )
-from pyield.bc.ptax import ptax_series
 from pyield.bc.repo import repos
 from pyield.bc.trades_intraday import tpf_intraday_trades
 from pyield.bc.trades_monthly import tpf_monthly_trades
@@ -20,6 +20,7 @@ __all__ = [
     "ptax_series",
     "repos",
     "tpf_monthly_trades",
+    "ptax",
     "tpf_intraday_trades",
     "selic_over",
     "selic_over_series",
